<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');
  
  /**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \par Common functions
   * \file
   *
   * Offers common functionality for elSQL.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */
  
  @include_once(dirname(__FILE__).'/../conf/conf.inc.php');
  @require_once(dirname(__FILE__).'/../language/language.inc.php');
  
  /**
   * Changes the user password of the logged in user.
   * \return Array with status information (ret: true or false, mes: message).
   */
  function changeUserPW()
  {
    global $pConf;

    $sOldPW = isset($_POST['opw']) ? trim($_POST['opw']) : '';
    $sNewPW = isset($_POST['npw']) ? trim($_POST['npw']) : '';
    $sNewPWValid = isset($_POST['npwv']) ? trim($_POST['npwv']) : '';
    $pReturn = array();
    
    //Password check
    if ($sNewPW == '' || $sNewPWValid == '')
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRPWBOTH;
      return $pReturn;
    }
    else if ($sNewPW != $sNewPWValid)
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRPWVAL;
      return $pReturn;
    }
    else if (strlen ($sNewPW) < 6)
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRPWLENG;
      return $pReturn;
    }
    
    if ($_SESSION['s_sPw'] != sha1($sOldPW))
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRPWCHANGEOLDPW;
      return $pReturn;
    }
    
    //Set new password
    @require_once(dirname(__FILE__).'/../database/db.inc.php');
    
    if($pDbCommands->changeUserPassword($pDb, $_SESSION['s_sEmail'], sha1($sNewPW)))
    {
      $pReturn['ret'] = true;
      $pReturn['mes'] = _PWCHANGESUCCESS;
      
      //Update the session
      $_SESSION['s_sPw'] = sha1($sNewPW);
    }
    else
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRPWCHANGE; 
    }
    
    return $pReturn;
  }

  /**
   * Resets a user database of the logged in user.
   * \return Array with status information (ret: true or false, mes: message).
   */
  function resetDB()
  {
    global $pConf;
    @require_once(dirname(__FILE__).'/../database/db.inc.php');
    
    $pReturn = array();
    
    if($pDbCommands->createTrainingData($pDb, $_SESSION['s_sUserID'], $_SESSION['s_sPw']))
    {
      $pReturn['ret'] = true;
      $pReturn['mes'] = _RESETDBSUCCESS;
    }
    else
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRRESETDB;
    }
    
    return $pReturn;
  }

  /**
   * Changes the user information of the logged in user.
   * \return Array with status information (ret: true or false, mes: message).
   */
  function changeUserInfo()
  {
    $sUserID = $_SESSION['s_sUserID'];
    $sFirstName = isset($_POST['firstname']) ? $_POST['firstname'] : '';
    $sLastName = isset($_POST['lastname']) ? $_POST['lastname'] : '';
    $sFieldOfStudy = isset($_POST['fieldofstudy']) ? $_POST['fieldofstudy'] : '';
    
    if (($sUserID == '') ||($sFirstName == '') || ($sLastName == '') || ($sFieldOfStudy == ''))
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRCHANGINGUSERFAILED;
      return false;
    }
    
    @require_once(dirname(__FILE__).'/../database/db.inc.php');
    
    //Security (escaping) handling
    $sUserID = $pDb->escapeString($sUserID);
    $sFirstName = $pDb->escapeString($sFirstName);
    $sLastName = $pDb->escapeString($sLastName);
    $sFieldOfStudy = $pDb->escapeString($sFieldOfStudy);
    
    //Change the user information
    if(!$pDbCommands->changeUserInfo($pDb, $sUserID, $sFirstName, $sLastName, $sFieldOfStudy))
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRCHANGINGUSERFAILED;
      return false;
    }
    
    $pReturn['ret'] = true;
    $pReturn['mes'] = _CHANGINGUSERSUCCESS;
    
    //Update the session
    $_SESSION['s_sLastName'] = $sLastName;
    $_SESSION['s_sFirstName'] = $sFirstName;
    $_SESSION['s_sFieldOfStudy'] = $sFieldOfStudy;
    
    return $pReturn;
  }

  /**
   * Setting new environment vairables and writ configuration file.
   * \return Array with status information (ret: true or false, mes: message).
   */
  function setEnvVar()
  {
    global $pConf;
 
    $sDBSystem = isset($_POST['dbsystem']) ? $_POST['dbsystem'] : $pConf['database']['system'];
    $sDBHost = isset($_POST['dbhost']) ? $_POST['dbhost'] : $pConf['database']['host'];
    $sDBPort = isset($_POST['dbport']) ? $_POST['dbport'] : $pConf['database']['port'];
    $sDBInstance = isset($_POST['dbinstance']) ? mb_strtolower($_POST['dbinstance'], mb_detect_encoding($_POST['dbinstance'])) : $pConf['database']['name'];
    $sDBAdminUser = isset($_POST['dbadminuser']) ? $_POST['dbadminuser'] : $pConf['database']['user'];
    $sDBAdminPwOld = isset($_POST['dbadminpwold']) ? $_POST['dbadminpwold'] : '';
    $sDBAdminPwNew = isset($_POST['dbadminpwnew']) ? $_POST['dbadminpwnew'] : '';
    $sDBAdminPwNewValid = isset($_POST['dbadminpwnewvalid']) ? $_POST['dbadminpwnewvalid'] : '';
    $sTitle = isset($_POST['title']) ? $_POST['title'] : $pConf['info']['title'];
    $sEmail = isset($_POST['email']) ? $_POST['email'] : $pConf['info']['email'];
    $sLanguage = isset($_POST['language']) ? $_POST['language'] : $pConf['common']['language'];
    $sDataDir = isset($_POST['datadir']) ? $_POST['datadir'] : $pConf['common']['datadir'];
    $sMaxNumResults = isset($_POST['maxnumresults']) ? $_POST['maxnumresults'] : $pConf['common']['maxnumresult'];
    $bEmailSupport = isset($_POST['emailsupport']) ? $_POST['emailsupport'] : $pConf['common']['emailsupport'];
    $sRegPassword = isset($_POST['regpassword']) ? $_POST['regpassword'] : $pConf['common']['regpassword'];
    $pReturn = array();
    
    //Check if a new administrator password has been submitted
    if ($sDBAdminPwNew != '')
    {
      //Check old password
      if ($pConf['database']['password'] != $sDBAdminPwOld)
      {
        $pReturn['ret'] = false;
        $pReturn['mes'] = _ERRPWCHANGEOLDPW;
        
        return $pReturn;
      }
      
      //Check validity of new password
      if ($sDBAdminPwNew != $sDBAdminPwNewValid)
      {
        $pReturn['ret'] = false;
        $pReturn['mes'] = _ERRPWVAL;
        
        return $pReturn;
      }
      
      @require_once(dirname(__FILE__).'/../database/db.inc.php');
      
      //Set new password
      if(!$pDbCommands->changeAdminPassword($pDb, $pConf['database']['user'], $sDBAdminPwNew))
      {
        $pReturn['ret'] = false;
        $pReturn['mes'] = _ERRCHANGEADMINPW;
        
        return $pReturn;
      }
      
      $sDBAdminPw = $sDBAdminPwNew;
    }
    else
    {
      $sDBAdminPw = $pConf['database']['password'];
    }
    
    //Write configuration file
    if (!writeConfig($sDBSystem, $sDBHost, $sDBPort, $sDBInstance, $sDBAdminUser, $sDBAdminPw, $sLanguage, $sDataDir, $sRegPassword, $sMaxNumResults, $bEmailSupport, $sTitle, $sEmail, false))
    {
    	  $pReturn['ret'] = false;
        $pReturn['mes'] = _ERRWRITECONFIGFILE;
        
        return $pReturn;
    }
    
    $pReturn['ret'] = true;
    $pReturn['mes'] = _SETENVSUCCESS;
    
    //Update session
    $_SESSION['s_sUserID'] = sha1($sDBAdminUser.$sDBAdminPw);
    
    return $pReturn;
  }

  /**
   * Resetting all user databases.
   * \return Array with status information (ret: true or false, mes: message).
   */
  function resetAllDB()
  {
    global $pConf;
    @require_once(dirname(__FILE__).'/../database/db.inc.php');
    
    $pReturn = array();
    
    if($pDbCommands->resetAllUserDb($pDb))
    {
      $pReturn['ret'] = true;
      $pReturn['mes'] = _RESETALLDBSUCCESS;
    }
    else
    {
      $pReturn['ret'] = false;
      $pReturn['mes'] = _ERRRESETALLDB;
    }
    
    return $pReturn;
  }
  
  /**
   * Returns the HTML code for the user administration table.
   * \param $sUserID User ID.
   * \param $sLastName Last name of the user.
   * \param $sFirstName First name of the user.
   * \param $sFieldOfStudy Field of study of the user.
   * \param $sEmail E-mail of the user.
   * \param $sLastLogin Date of last login.
   * \return Echo HTML code.
   */
  function echoUserTable($sUserID, $sLastName, $sFirstName, $sFieldOfStudy, $sEmail, $sLastLogin)
  {
    echo '            <tr>
                        <td class="useradmin"><input type="checkbox" name="seluser[]" value="'.$sUserID.'" /></td>
                        <td class="useradmin"><a href="admin.php?act=edius&id='.$sUserID.'&" '."onclick=\"return confirmAction('e','".$sLastName." ".$sFirstName."'".');"><img class="icon" src="../img/edit.png" alt="'._EDIT.'" /></a></td>
                        <td class="useradmin"><a href="admin.php?act=delus&id='.$sUserID.'&" '."onclick=\"return confirmAction('d','".$sLastName." ".$sFirstName."'".');"><img class="icon" src="../img/delete.png" alt="'._DELETE.'" /></a></td>
                        <td class="useradmin">'.$sLastName.' '.$sFirstName.'</td>
                        <td class="useradmin">'.$sFieldOfStudy.'</td>
                        <td class="useradmin">'.$sEmail.'</td>
                        <td class="useradmin">'.$sLastLogin.'</td>
                      </tr>
         ';
  }

  /**
   * Writes the configuration file.
   * \param $sDBSystem Database system identifier (e.g. "mysql", "pgsql" a.s.o.).
   * \param $sDBHost Database host.
   * \param $sDBPort Port on which database is listening.
   * \param $sDBInstance Name of the database instance.
   * \param $sDBAdminUser Database and elSQL administrator user name.
   * \param $sDBAdminPw Database and elSQL administrator password.
   * \param $sLanguage Selected frontend language.
   * \param $sDataDir Data directory.
   * \param $sRegPassword Registration password.
   * \param $sMaxNumResults Maximum number of shown rows in result sets.
   * \param $bEmailSupport True if server is capable to send e-mails, otherwise
   *        false.
   * \param $sTitle Title of this application, normally the university name.
   * \param $sEmail Administrator e-mail address.
   * \param $bNewFile Set true to create a new file, false to edit the existing
   *        file.
   * \return True on success, otherwise false.
   */
  function writeConfig($sDBSystem, $sDBHost, $sDBPort, $sDBInstance, $sDBAdminUser, $sDBAdminPw, $sLanguage, $sDataDir, $sRegPassword, $sMaxNumResults, $bEmailSupport, $sTitle, $sEmail, $bNewFile)
  {
    if ($bNewFile)
    {
      $pFile = fopen(dirname(__FILE__).'/../conf/conf.inc.php', 'w');
    }
    else
    {
      $pFile = fopen(dirname(__FILE__).'/../conf/conf.inc.php', 'w');
    }
    
    if (!$pFile)
      return false;
    
    fwrite($pFile, "<?php\n");
    fwrite($pFile, "defined('_ACCESS_') or die('Direct Access to this location is not allowed.');\n");
    fwrite($pFile, "\n");
    fwrite($pFile, "\$pConf['database']['system']     = '".$sDBSystem."';\n");
    fwrite($pFile, "\$pConf['database']['host']       = '".$sDBHost."';\n");
    fwrite($pFile, "\$pConf['database']['port']       = '".$sDBPort."';\n");
    fwrite($pFile, "\$pConf['database']['name']       = '".$sDBInstance."';\n");
    fwrite($pFile, "\$pConf['database']['user']       = '".$sDBAdminUser."';\n");
    fwrite($pFile, "\$pConf['database']['password']   = '".$sDBAdminPw."';\n");
    fwrite($pFile, "\n");
    fwrite($pFile, "\$pConf['common']['language']     = '".$sLanguage."';\n");
    fwrite($pFile, "\$pConf['common']['datadir']      = '".$sDataDir."';\n");
    fwrite($pFile, "\$pConf['common']['regpassword']  = '".$sRegPassword."';\n");
    fwrite($pFile, "\$pConf['common']['maxnumresult'] = '".$sMaxNumResults."';\n");
    fwrite($pFile, "\$pConf['common']['emailsupport'] = ".$bEmailSupport.";\n");
    fwrite($pFile, "\n");
    fwrite($pFile, "\$pConf['info']['title']          = '".$sTitle."';\n");
    fwrite($pFile, "\$pConf['info']['email']          = '".$sEmail."';\n");
    fwrite($pFile, "\n");
    fwrite($pFile, "/* Don't change values below here! */\n");
    fwrite($pFile, "\$pConf['internal']['version']    = '0.9.4';\n");
    fwrite($pFile, "?>");
    
    fclose($pFile);
    
    return true;
  }
