<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');

	/**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \class sqlresult
   *
   * Offers functionality for sqlresult.php.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */
  
  @require_once('./conf/conf.inc.php');
  @require_once('./language/language.inc.php');

  class sqlresult
  {
    /**
     * Constructor
     */
    public function __construct()
    {
    }

    /**
     * Destructor
     */
    public function __destruct()
    {
    }
    
    /**
     * Execute SQL statement and prepare the result for output (includes in
     * addition to the result of the statement also the log creation).
     * \param $sSQL SQL statement.
     * \param $sUser User ID.
     * \param $sPassword Password of the user.
     * \param $bLog Set to true if a log should be generated.
     * \param $bShowAll Set to true if the result should contain all the data
     *        (without concerning the restirction of maximum number of rows in
     *        result).
     * \return Array with formatted SQL result, SQL statement and if selected
     *         the log entry (res: SQL result, log: log entry).
     */
    public function showResult($sSQL, $sUser, $sPassword, $bLog, $bShowAll)
    {
      @require_once('./database/db.inc.php');
      
      global $pConf;
      
      $pReturn = array();
      $sTemp = '';
      
      //Print SQL statement to log
      if ($bLog)
      {
        $pReturn['log'] = $sSQL."\n";
      }
      
      //Execute SQL
      $pDb->connectDBUser($sUser, $sPassword);
      
      //On error send database error to user
      if(!$pDb->executeSQL($sSQL))
      {
        $sTemp = $pDb->getLastError();
        $pReturn['res'] = "<div class=\"error\">"._ERRQUERYFAILED.trim($sTemp)."</div>";
          
        if ($bLog)
        {
          $pReturn['log'] .= $sTemp."\n\n";
        }
      }
      else
      {
        //Format SQL result
        $pReturn['res'] =  "<table class=\"result\">\n";
        $pReturn['res'] .= "<tr>\n";
          
        $iRecordCount = 1;
        $iNumRows = $pDb->getNumberRows();
        $iNumFields = $pDb->getNumberFields();
        $iNumAffectedRows = $pDb->getAffectedRows();
        $iNumField = 1;
        
        if($iNumRows)
        {
          //Show rows (select query)
          while($pDb->hasMoreFields())
          {
            //Show field names of the table
            $pTemp = $pDb->getNextFieldDescription();
            $pTemp[0] = trim($pTemp[0]);
            
            $pReturn['res'] .= "<th class=\"result\">".$pTemp[0]."</th>\n";
            $pFieldLength[$iNumField] = $pTemp[1];
            
            if ($bLog)
            {
              $pReturn['log'] .= sprintf("%-".$pFieldLength[$iNumField]."s ", $pTemp[0]);
            }
            
            $iNumField++;
          }
            
          $pReturn['res'] .="</tr>\n";
          
          //Create separation of description and data in log
          if ($bLog)
          {
            $pReturn['log'] .= "\n";
            
            for($i = 1; $i <= $iNumFields; $i++)
            {
              $pReturn['log'] .= sprintf("%'-".$pFieldLength[$i]."s ", "");
            }
            $pReturn['log'] .= "\n";
          }
          
          //Show data
          while(($pDb->getNextRecord()) && (($bShowAll) || ($iRecordCount <= $pConf['common']['maxnumresult'])))
          {
            $pReturn['res'] .= "<tr>\n";
            
            $iNumField = 1;
            
            while($pDb->hasMoreFields())
            {
              $sTemp = trim($pDb->getNextField());
              $pReturn['res'] .= "<td class=\"result\">".$sTemp."</td>";
              
              if ($bLog)
              {
                $pReturn['log'] .= sprintf("%-".$pFieldLength[$iNumField]."s ", $sTemp);
                
                $iNumField++;
              }
            }
            
            $pReturn['res'] .= "</tr>\n";
            
            if ($bLog)
            {
              $pReturn['log'] .= "\n";
            }
            $iRecordCount = $iRecordCount + 1;
          }
        
          $pReturn['res'] .= "</table>\n";
          
          //Log: show number rof selected rows
          //Log: if not all results could have been displayed create message
          if ($bLog)
          {
            if ($bShowAll || ($iNumRows <= $pConf['common']['maxnumresult']))
            {
              $pReturn['log'] .= "\n".$iNumRows." "._NUMROWSSELECTED;
              $pReturn['log'] .= "\n\n";
            }
            else
            {
              $pReturn['log'] .= "\n".$pConf['common']['maxnumresult']." "._NUMROWSSELECTEDOF." ".$iNumRows." "._NUMROWSSELECTED."\n"._NUMROWSSELECTEDMAXEXCEED;
              $pReturn['log'] .= "\n\n";  
            }
          }
          
          //Result: if not all results could have been displayed create message
          if (!$bShowAll && ($iNumRows > $pConf['common']['maxnumresult']))
          {
            $pReturn['res'] .= "<br /><div class=\"message\">"._NUMROWSSELECTEDMAXEXCEED."</div>\n";
          }
        }
        else
        {
          //Affected rows (not a select query)
          $pReturn['res'] .= "<br /><div class=\"message\">".$iNumAffectedRows." "._NUMROWSAFFECTED."</div>\n";
            
          if ($bLog)
          {
            $pReturn['log'] .= "\n".$iNumAffectedRows." "._NUMROWSAFFECTED;
            $pReturn['log'] .= "\n\n";
          }
        }
      }
      
      $pDb->closeDB();
      
      return $pReturn;
    }
    
    /**
     * Formatting SQL statement with GeSHi.
     * \param $sSQL SQL statement.
     * \return Formatted SQL statement.
     */
    public function formatStatement($sSQL)
    {
      @require_once('./lib/geshi/geshi.php');
      
      global $pConf;
      
      //Decide which language, per default SQL is used
      $sLanguage = 'sql';
      
      if ($pConf['database']['system'] == 'mysql')
      {
        $sLanguage = 'mysql';
      }
      else if ($pConf['database']['system'] == 'oracle')
      {
        $sLanguage = 'oracle8';
      }
      
      //Use GeSHi
      $pGeshi = new GeSHi($sSQL, $sLanguage);
      $pGeshi->set_case_keywords(GESHI_CAPS_UPPER);
      
      return $pGeshi->parse_code();
    }
  }
?>