<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');

  /**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \interface iDatabase
   *
   * Defines methodes which must be supported by a implemented database system.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */
	
  interface iDatabase
  {
    /**
     * Connect to database.
     * \param $sHost
     * \param $sPort
     * \param $sDBName
     * \param $sUser
     * \param $sPassword
     * \return True on success, otherwise false.
     */
    public function connectDB($sHost, $sPort, $sDBName, $sUser, $sPassword);
		/**
		 * Connect administrator to database.
		 * \return True on success, otherwise false.
		 */
    public function connectDBAdmin();
		/**
		 * Connect user to database.
		 * \param $sUser
     * \param $sPassword
		 * \return True on success, otherwise false.
		 */
    public function connectDBUser($sUser, $sPassword);
		/**
		 * Close database connection.
		 * \return True on success, otherwise false.
		 */
    public function closeDB();
		/**
		 * Executes a transaction (array with SQL statements).
		 * \param $pQueryArray
		 * \return True on success, otherwise false.
		 */
    public function executeTransaction($pQueryArray);
		/**
		 * Executes a single query.
		 * \param $sQuery
		 * \return True on success, otherwise false.
		 */
    public function executeSQL($sQuery);
		/**
		 * Get next record, free result set if no more recors are available.
		 * \return True if there are more records, otherwise false.
		 */
    public function getNextRecord();
		/**
		 * Check if there are more fields in the result set.
		 * \return True if there are more fields, otherwise false.
		 */
    public function hasMoreFields();
		/**
		 * Get next field data.
		 * \return Data.
		 */
    public function getNextField();
		/**
		 * Get next field description.
		 * \return Array with the field name and with the lenght of the field in the
		 *         database or, if this is not available the length of the name.
		 */
    public function getNextFieldDescription();
		/**
		 * Get number of affected rows by insert, update a.s.o.
		 * \return Number of affected rows.
		 */
    public function getAffectedRows();
		/**
		 * Get number of rows in result set.
		 * \return Number of rows.
		 */
    public function getNumberRows();
		/**
		 * Get number of fields in result set.
		 * \return Number of fields.
		 */
    public function getNumberFields();
		/**
		 * Get last database error.
		 * \return Database error as string.
		 */
    public function getLastError();
		/**
		 * Get last database error number.
		 * \return Database error number.
		 */
    public function getLastErrorNo();
		/**
		 * Escapes string for security reasons.
		 * \param $sString String to escape.
		 * \return Escaped string.
		 */
    public function escapeString($sString);
  }
?>