<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');
  
  /**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \interface iDatabaseCommands
   *
   * Defines database commands which must be supported by a implemented database system.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */

  interface iDatabaseCommands
  {
		/**
     * Creates a new user.
     * \param $pDb
     * \param $sFirstName
     * \param $sLastName
     * \param $sFieldOfStudy
     * \param $sEmail
     * \param $sPw
     * \return True on success, otherwise false.
     */
    public function createNewUser($pDb, $sFirstName, $sLastName, $sFieldOfStudy, $sEmail, $sPw);
		/**
     * Delete a user.
     * \param $pDb
     * \param $sUserID
     * \return True on success, otherwise false.
     */
		public function deleteUser($pDb, $sUserID);
		/**
     * Change user password.
     * \param $pDb
     * \param $sUserEmail
     * \param $sNewPw
     * \return True on success, otherwise false.
     */
		public function changeUserPassword($pDb, $sUserEmail, $sNewPw);
		/**
     * Change administrator password.
     * \param $pDb
     * \param $sAdminName
     * \param $sNewPw
     * \return True on success, otherwise false.
     */
		public function changeAdminPassword($pDb, $sAdminName, $sNewPw);
		/**
     * Creates training data or resets user database to initial values.
     * \param $pDb
     * \param $sUserID
     * \param $sPw
     * \return True on success, otherwise false.
     */
		public function createTrainingData($pDb, $sUserID, $sPw);
		/**
     * Get login data by user name (e-mail address)
     * \param $pDb
     * \param $sUserName
     * \return Array with login data (0: User ID, 1: Last name, 2: First name,
     *         3: Password, 4: Field of study, 5: E-mail address) on success,
     *         otherwise false.
     */
		public function getLoginData($pDb, $sUserName);
		/**
     * Get login data by user ID.
     * \param $pDb
     * \param $sUserID
     * \return Array with login data (0: User ID, 1: Last name, 2: First name,
     *         3: Password, 4: Field of study, 5: E-mail address) on success,
     *         otherwise false.
     */ 
		public function getLoginDataById($pDb, $sUserID);
 		/**
     * Set login date (format Y-M-D, e.g. 2007-10-14) of user (actual date used,
     * according to server date).
     * \param $pDb
     * \param $sUserName
     */
		public function setLastLoginDate($pDb, $sUserName);
 		/**
     * Reset all user databases.
     * \param $pDb
     * \return True on success, otherwise false.
     */
		public function resetAllUserDb($pDb);
 		/**
     * Build table for user administration.
     * \param $pDb
     * \return True on success, otherwise false.
     */
		public function buildUserTable($pDb);
		/**
     * Change user information.
     * \param $pDb
     * \param $sUserID
     * \param $sFirstName
     * \param $sLastName
     * \param $sFieldOfStudy
     * \return True on success, otherwise false.
     */
		public function changeUserInfo($pDb, $sUserID, $sFirstName, $sLastName, $sFieldOfStudy);
    /**
     * Builds table which shows the actual data in the database.
     * \param $pDb Pointer to database connection.
     * \param $sUserID User ID.
     * \param $sPw Password.
     * \return True on success, otherwise false.
     */    
    public function showActualDataUserDB($pDb, $sUserID, $sPw);
  }
?>