<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');

  /**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \class mysql
   *
   * mySQL database handler.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */
  
  @include_once(dirname(__FILE__).'/../conf/conf.inc.php');
  @require_once(dirname(__FILE__).'/idatabase.class.php');
  
  class mysql implements iDatabase
  {
    //Member variables
    private $iLinkID       = 0;
    private $iQueryID      = 0;
    private $pRecord       = array();
    private $iField        = 0;
    private $iErrorNo      = 0;
    private $sError        = "";
    private $iNumberFields = -1;
    private $iNumberRows   = -1;
    
    /**
     * Constructor 
     */
    public function __construct()
    {
    }
    
    /**
     * Destructor 
     */
    public function __destruct()
    {
      $this->closeDB();
    }
    
    /**
     * Connects to the database server.
     * \param $sHost Hostname or IP adress of database server.
     * \param $sPort Port of database server.
     * \param $sDBName Database instance name.
     * \param $sUser User name to connect.
     * \param $sPassword Password to user name.
     * \return True on success, otherwise false.
     */
    public function connectDB($sHost, $sPort, $sDBName, $sUser, $sPassword)
    {      
      if (!($this->iLinkID = @mysqli_connect($sHost, $sUser, $sPassword, $sDBName, $sPort)))
      {
        $this->iErrorNo = @mysqli_connect_errno($this->iLinkID);
        $this->sError = @mysqli_connect_error($this->iLinkID);
        return false;
      }
      
      return true;
    }
    
		/**
		 * Connect administrator to database.
		 * \return True on success, otherwise false.
		 */
    public function connectDBAdmin()
    {
      global $pConf;
      
      return $this->connectDB($pConf['database']['host'], $pConf['database']['port'], $pConf['database']['name'], $pConf['database']['user'], $pConf['database']['password']);
    }
    
		/**
		 * Connect user to database.
		 * \param $sUser User name.
     * \param $sPassword Password.
		 * \return True on success, otherwise false.
		 */
    public function connectDBUser($sUser, $sPassword)
    {
      global $pConf;
      
      return $this->connectDB($pConf['database']['host'], $pConf['database']['port'], $pConf['database']['name'].'_'.$sUser, $sUser, $sPassword);
    }
    
		/**
		 * Frees result and close the database connection.
		 * \return True on success, otherwise false.
		 */
    public function closeDB()
    {
      if ($this->iQueryID)
        $this->freeResult();
      
      if (!@mysqli_close($this->iLinkID))
        return false;
      
      $this->iLinkID = 0;
      
      return true;
    }
    
		/**
		 * Executes a transaction (array with SQL statements).
		 * \param $pQueryArray
		 * \return True on success, otherwise false.
		 */
    public function executeTransaction($pQueryArray)
    {
      //Set database to transaction mode
      @mysqli_autocommit($this->iLinkID, false);
      
      foreach ($pQueryArray as $sSingleQuery)
        $this->executeSQL($sSingleQuery);
      
      $bSuccess = @mysqli_commit($this->iLinkID);
      
      //If a statement failed, roll back the database
      if (!$bSuccess)
        @mysqli_rollback($this->iLinkID);
      
      //Set database to no-transaction mode
      @mysqli_autocommit($this->iLinkID, true);
      
      return $bSuccess;
    }
    
		/**
		 * Executes a single query.
		 * \param $sQuery
		 * \return True on success, otherwise false.
		 */
    public function executeSQL($sQuery)
    {
      if ($sQuery == '')
        return false;
      
      //If there is an old query, free result first
      if ($this->iQueryID)
        $this->freeResult();
      
      $this->iQueryID = @mysqli_query($this->iLinkID, $sQuery);
      
      $this->iNumberFields = @mysqli_num_fields($this->iQueryID);
      $this->iNumberRows = @mysqli_num_rows($this->iQueryID);
      
      if (!$this->iQueryID)
      {
        $this->iErrorNo = @mysqli_errno($this->iLinkID);
        $this->sError = @mysqli_error($this->iLinkID);
        return false;
      }
      
      return true;
    }
    
 		/**
		 * Get next record, free result set if no more recors are available.
		 * \return True if there are more records, otherwise false.
		 */
    public function getNextRecord()
    {
      //Set field count ot Zero
      $this->iField = 0;
      
      //Get next record
      $this->pRecord = @mysqli_fetch_array($this->iQueryID);
      
      //Check for errors
      $this->iErrorNo = @mysqli_errno($this->iLinkID);
      $this->sError = @mysqli_error($this->iLinkID);
      
      $bStatus = @is_array($this->pRecord);
      
      //If there are no more records, free result
      if (!$bStatus)
      {
        $this->freeResult();
      }
      
      return $bStatus;
    }
    
		/**
		 * Check if there are more fields in the result set.
		 * \return True if there are more fields, otherwise false.
		 */
    public function hasMoreFields()
    {
      if (($this->iField + 1) > $this->iNumberFields)
      {
        $this->iField = 0;
        return false;
      }
      
      return true;
    }
    
		/**
		 * Get next field data.
		 * \return Data.
		 */
    public function getNextField()
    {
      return $this->pRecord[$this->iField++];
    }
    
		/**
		 * Get next field description.
		 * \return Array with the field name and with the lenght of the field in the
		 *         database or, if this is not available the length of the name.
		 */
    public function getNextFieldDescription()
    {
      //Get description
      $pReturn[0] = @mysqli_fetch_field_direct($this->iQueryID,$this->iField)->name;
      
      //Length of name and lenght of field in database
      $pReturn[1] = strlen($pReturn[0]);
      $iTempFieldLength = @mysqli_fetch_field_direct($this->iQueryID,$this->iField)->length;
      
      //Use the longer one
      if ($pReturn[1] < $iTempFieldLength)
        $pReturn[1] = $iTempFieldLength;

      $this->iField++;
      
      return $pReturn;
    }
    
		/**
		 * Get number of affected rows by insert, update a.s.o.
		 * \return Number of affected rows.
		 */
    public function getAffectedRows()
    {
      if (!$this->iQueryID)
        return -1;
      
      return @mysqli_affected_rows($this->iLinkID);
    }
    
		/**
		 * Get number of rows in result set.
		 * \return Number of rows.
		 */
    public function getNumberRows()
    {
      if (!$this->iQueryID)
        return -1;

      return $this->iNumberRows;
    }
    
		/**
		 * Get number of fields in result set.
		 * \return Number of fields.
		 */
    public function getNumberFields()
    {
      if (!$this->iQueryID)
        return -1;
      
      return $this->iNumberFields;
    }
    
    /**
		 * Get last database error.
		 * \return Database error as string.
		 */
    public function getLastError()
    {
      return $this->sError;
    }
    
    /**
		 * Get last database error number.
		 * \return Database error number.
		 */
    public function getLastErrorNo()
    {
      return $this->iErrorNo;
    }
    
    /**
		 * Escapes string for security reasons.
		 * \param $sString String to escape.
		 * \return Escaped string.
		 */
    public function escapeString($sString)
    {
      //If there is no database connection make a connection
      if(!$this->iLinkID)
      {
        $this->connectDBAdmin();
        $sString = @mysqli_real_escape_string($this->iLinkID, $sString);
        $this->closeDB();
        
        return $sString;
      }
      else
      {
        return @mysqli_real_escape_string($this->iLinkID, $sString);
      }
    }
    
    /**
		 * Free result set.
		 */
    private function freeResult()
    {
      @mysqli_free_result($this->iQueryID);
      $this->iQueryID = 0;
      $this->iField = 0;
    }
  }
?>