<?php
  //No direct access
  defined('_ACCESS_') or die('Direct Access to this location is not allowed.');

  /**
	 * <!-- elSQL - e-learn SQL -->
   *
   * \class pgsql
   *
   * PostgreSQL database handler.
   *
   * \author Stephan Schuetz, stephan.schuetz(at)fhnw.ch
   * \date 14.10.2007
   * \version 1.0
   *
   * (c) Copyright GITTA 2007. Creative Commons License "Attribution-Noncommercial-Share Alike"
	 *                           http://creativecommons.org/licenses/by-nc-sa/3.0/
   */
   
  @include_once(dirname(__FILE__).'/../conf/conf.inc.php');
  @require_once(dirname(__FILE__).'/idatabase.class.php');
  
  class pgsql implements iDatabase
  {
    //Member variables
    private $iLinkID       = 0;
    private $iQueryID      = 0;
    private $pRecord       = array();
    private $iRow          = 0;
    private $iField        = 0;
    private $iErrorNo      = 0;
    private $sError        = "";
    private $iNumberFields = -1;
    private $iNumberRows   = -1;
    
    /**
     * Constructor 
     */
    public function __construct()
    {
    }
    
    /**
     * Destructor 
     */
    public function __destruct()
    {
      $this->closeDB();
    }
		
    /**
     * Connects to the database server.
     * \param $sHost Hostname or IP adress of database server.
     * \param $sPort Port of database server.
     * \param $sDBName Database instance name.
     * \param $sUser User name to connect.
     * \param $sPassword Password to user name.
     * \return True on success, otherwise false.
     */
    public function connectDB($sHost, $sPort, $sDBName, $sUser, $sPassword)
    {
      $sConnect = 'host='.$sHost.' port='.$sPort.' dbname='.$sDBName.' user='.$sUser.' password='.$sPassword;
      
      if (!($this->iLinkID = @pg_connect($sConnect)))
      {
        $this->sError = @pg_last_error($this->iLinkID);
        $this->iErrorNo = (($this->sError == '') ? 0 : 1);
        return false;
      }
       
      return true;
    }
		
		/**
		 * Connect administrator to database.
		 * \return True on success, otherwise false.
		 */
    public function connectDBAdmin()
    {
      global $pConf;
      
      return $this->connectDB($pConf['database']['host'], $pConf['database']['port'], $pConf['database']['name'], $pConf['database']['user'], $pConf['database']['password']);
    }
    
		/**
		 * Connect user to database.
		 * \param $sUser User name.
     * \param $sPassword Password.
		 * \return True on success, otherwise false.
		 */
    public function connectDBUser($sUser, $sPassword)
    {
      global $pConf;
      
      return $this->connectDB($pConf['database']['host'], $pConf['database']['port'], $pConf['database']['name'], $sUser, $sPassword);
    }
    
		/**
		 * Frees result and close the database connection.
		 * \return True on success, otherwise false.
		 */
    public function closeDB()
    {
      if ($this->iQueryID)
        $this->freeResult();
      
      if (!@pg_close($this->iLinkID))
        return false;
      
      $this->iLinkID = 0;
      
      return true;
    }

		/**
		 * Executes a transaction (array with SQL statements).
		 * \param $pQueryArray
		 * \return True on success, otherwise false.
		 */
    public function executeTransaction($pQueryArray)
    {
      $sQuery = 'BEGIN;';
      
      foreach ($pQueryArray as $sSingleQuery)
        $sQuery .= $sSingleQuery.';';
      
      $sQuery .= 'COMMIT;';

      return $this->executeSQL($sQuery);
    }
		
		/**
		 * Executes a single query.
		 * \param $sQuery
		 * \return True on success, otherwise false.
		 */
    public function executeSQL($sQuery)
    {
      @pg_set_client_encoding($this->iLinkID, mb_internal_encoding());
      
      if ($sQuery == '')
        return false;
      
			//If there is an old query, free result first
      if ($this->iQueryID)
        $this->freeResult();

      $this->iQueryID = @pg_query($this->iLinkID, $sQuery);

      $this->iNumberFields = @pg_num_fields($this->iQueryID);
      $this->iNumberRows = @pg_num_rows($this->iQueryID);

      if (!$this->iQueryID)
      {
        $this->sError = @pg_last_error($this->iLinkID);
        $this->iErrorNo = (($this->sError == "") ? 0 : 1);
        return false;
      }
      
      return true;
    }

 		/**
		 * Get next record, free result set if no more recors are available.
		 * \return True if there are more records, otherwise false.
		 */
    public function getNextRecord()
    {
      //Set field count ot Zero
      $this->iField = 0;
      
			//Get next record
      $this->pRecord = @pg_fetch_array($this->iQueryID);
      
			//Check for errors
      $this->sError = @pg_last_error($this->iLinkID);
      $this->iErrorNo = (($this->sError == "") ? 0 : 1);
      
      $bStatus = is_array($this->pRecord);
      
			//If there are no more records, free result
      if (!$bStatus)
      {
        $this->freeResult();
      }
      
      return $bStatus;
    }
		
		/**
		 * Check if there are more fields in the result set.
		 * \return True if there are more fields, otherwise false.
		 */
    public function hasMoreFields()
    {
      if (($this->iField + 1) > $this->iNumberFields)
      {
        $this->iField = 0;
        return false;
      }
      
      return true;
    }
    
		/**
		 * Get next field data.
		 * \return Data.
		 */
    public function getNextField()
    {
      return $this->pRecord[$this->iField++];
    }
		
		/**
		 * Get next field description.
		 * \return Array with the field name and with the lenght of the field in the
		 *         database or, if this is not available the length of the name.
		 */
    public function getNextFieldDescription()
    {
			//Get description
      $pReturn[0] = @pg_field_name($this->iQueryID, $this->iField);
      
			//Length of name
      $pReturn[1] = strlen($pReturn[0]);
      
			//Get length of longest field in result set, if it is longer than the
			//description use this lenght
      for ($i = 0; $i < $this->iNumberRows; $i++)
      {
        $iTempFieldLength = @pg_field_prtlen($this->iQueryID, $i, $this->iField);
        
        if ($pReturn[1] < $iTempFieldLength)
          $pReturn[1] = $iTempFieldLength;
      }
      
      $this->iField++;
      
      return $pReturn;
    }
    
		/**
		 * Get number of affected rows by insert, update a.s.o.
		 * \return Number of affected rows.
		 */
    public function getAffectedRows()
    {
      if (!$this->iQueryID)
        return -1;
			
      return @pg_affected_rows($this->iQueryID);
    }
		
		/**
		 * Get number of rows in result set.
		 * \return Number of rows.
		 */
    public function getNumberRows()
    {
      if (!$this->iQueryID)
        return -1;
			
      return $this->iNumberRows;
    }
		
		/**
		 * Get number of fields in result set.
		 * \return Number of fields.
		 */
    public function getNumberFields()
    {
      if (!$this->iQueryID)
        return -1;
      
      return $this->iNumberFields;
    }
    
		/**
		 * Get last database error.
		 * \return Database error as string.
		 */
    public function getLastError()
    {
      return $this->sError;
    }
    
		/**
		 * Get last database error number.
		 * \return Database error number.
		 */
    public function getLastErrorNo()
    {
      return $this->iErrorNo;
    }
		
    /**
		 * Escapes string for security reasons.
		 * \param $sString String to escape.
		 * \return Escaped string.
		 */
    public function escapeString($sString)
    {
      return @pg_escape_string($sString);
    }
		
    /**
		 * Free result set.
		 */
    private function freeResult()
    {
      @pg_free_result($this->iQueryID);
      $this->iQueryID = 0;
      $this->iRow = 0;
      $this->iField = 0;
    }
  }
?>